
#include "ScriptManager.h"
#include "CondorManager.h"
#include "StringManager.h"

using namespace std;
using namespace CloudResourceManager;


StringVector CondorManager::get_ip()
{
    const char* cmd = "/usr/bin/condor_status -startd -format \"%s\\n\" StartdIpAddr";
    string line = ScriptManager::runscript(cmd);
    return StringManager::split(line, " \t\n<>");
}


bool CondorManager::submit_job(const string& jobName, string& condorId)
{
    const char* exeName = "/usr/bin/condor_submit";
    bool result = false;
    StringVector lines = ScriptManager::runscript(exeName, jobName);
    if (lines.size() == 3)
    {
        StringVector words = StringManager::split(lines[2]);
        if (words.size() == 6)
        {
            if (words[2].compare("submitted") == 0)
            {
                result = true;
                condorId = words[5];
            }
        }
    }
    return result;
}

bool CondorManager::reconfig(const string& address)
{
    string cmd = "/usr/sbin/condor_reconfig -addr " + address + " -startd";
    string line = ScriptManager::runscript(cmd);
    if (line.substr(0, 4).compare("Sent") == 0)
    {
        return true;
    }
    return false;
}


bool CondorManager::set_variable_value(const string& ip, const string& var, const string& value)
{
    int i, j;
    StringVector ipVector = get_ip();
    for (i = 0; i < ipVector.size(); i ++)
    {
        StringVector ipPortVector= StringManager::split(ipVector[i], ":");
        if (ipPortVector[0].compare(ip) == 0)
        {
            // we still need the format of "<ip:port>
            string address = "\"<" + ipVector[i] + ">\"";
            string cmd = "/usr/bin/condor_config_val -address " + address;
            cmd.append(" -startd -set ");
            cmd.append(var);
            cmd.append("=");
            cmd.append(value);
            string line = ScriptManager::runscript(cmd);
            FILE_LOG(logDEBUG)<<"set_variable_value:"<<line;
            StringVector words = StringManager::split(line);
            for (j = 0; j < words.size(); j ++)
            {
                if (StringManager::incompare(words[j], string("success"), sizeof("success")) == 0)
                {
                    return reconfig(address);
                }
            }
        }
    }
    return false;
}


bool CondorManager::check_active_instance(const string& instanceIp)
{
    const char* cmd = "/usr/bin/condor_status -master -format \"%s\\n\" MasterIpAddr";
    string line = ScriptManager::runscript(cmd);
    return line.find(instanceIp)!= string::npos;
}

// -1 indicates not found
int CondorManager::find_job_array(const string& jobId, string& status)
{
    string runtime;
    int result = -1;
    bool bFound = false;
    StringVector results = ScriptManager::runscript("/usr/bin/condor_q", jobId);
    if (results.size() >= 5)
    {
        //successful results are 5 lines, this is NOT very good, but let us improve it later...
        StringVector words = StringManager::split(results[4]);
        if (words[0].compare(jobId) == 0)
        {
            bFound = true;
            runtime= words[4];
            status = words[5];
        }
    }
    else
    {
        if (results.size()== 4)
        {
            results = ScriptManager::runscript("/usr/bin/condor_history", jobId);
            if (results.size() >= 2)
            {
                StringVector words = StringManager::split(results[1]);
                if (StringManager::incompare(jobId, words[0], jobId.size()) == 0)
                {
                    bFound = true;
                    runtime= words[4];
                    status = words[5];
                }
            }
        }
    }
    if (bFound)
    {
        int day = 0, hour = 0, minute = 0, second = 0;
        if (sscanf(runtime.c_str(), "%d+%d:%d:%d", &day, &hour, &minute, &second) == 4)
        {
            result = day*86400 + hour*3600 + minute*60 + second;
        }
    }
    return result;
}


StringVector CondorManager::find_idle_ip()
{
    const char* cmd = "/usr/bin/condor_status";
    const char* param = "-startd -format \"%20.20s\\t\" StartdIpAddr -format \"%s\\n\" activity";
    StringVector result;
    StringMap statusMap;
    StringMap::iterator it;

    StringVector lines = ScriptManager::runscript(cmd, param);
    for (int i = 0; i < lines.size(); i ++)
    {
        if (lines[i].size()> 0)
        {
            StringVector words = StringManager::split(lines[i], " \t\n<>:");
            if (words.size() >= 3)
            {
                it = statusMap.find(words[0]);
                if (it == statusMap.end())
                {
                    statusMap.insert(StringStringPair(words[0], words[2]));
                }
                else
                {
                    // we only need to update when it is NOT idle
                    if (words[2].compare("Idle") != 0)
                    {
                        // update the status
                        it->second = words[2];
                    }
                }
            }
        }
    }
    for (it = statusMap.begin(); it != statusMap.end(); ++it)
    {
        if (it->second.compare("Idle") == 0)
        {
            result.push_back(it->first);
        }
    }
    return result;
}


bool CondorManager::config_taskId(const std::string& ip, const string& taskId)
{
    return set_variable_value(ip, "TaskId", taskId);
}

bool CondorManager::config_taskStatus(const std::string& ip, const string& taskStatus)
{
    return set_variable_value(ip, "Task_Status", taskStatus);
}

